/**
 * Fast Matrix Exponentiation
 *
 * Complexity (Time): O(K^3 log n)
 * Complexity (Space): O(K^2)
 */

// This algorithm is used to solve recurrences such as:
//   f(n) = x1 * f(n - 1) + x2 * f(n - 1) + ... + xk * f(n - k)
//
// It works by defining this recurrence as a linear combination,
// for example (k = 2):
//   f(n) = [x1  x2] [f(n - 1)]
//                   [f(n - 2)]
// It can be rewriten as:
//   [  f(n)  ] = [x1  x2] [f(n - 1)]
//   [f(n - 1)]   [ 1   0] [f(n - 2)]
//
// And that is solved by calculating the following matrix power:
//   [x1  x2]^n
//   [ 1   0]


#define K 2

struct matrix {
  ll m[K][K];

  // Matrix multiplication - O(k^3)
  matrix operator*(matrix a) {
    matrix aux;

    for (int i = 0; i < K; i++)
      for (int j = 0; j < K; j++) {
        ll sum = 0;

        for (int k = 0; k < K; k++)
          sum += (m[i][k] * a[k][j]) % MOD;

        aux[i][j] = sum % MOD;
      }
    
    return aux;
  }

  ll *operator[](int i) {
    return m[i];
  }

  void clear() {
    mset(m, 0);
  }
};


// Fast exponentiation (can be used with integers as well) - O(log n)
matrix matrix_pow(matrix in, ll n) {
  matrix ans, b = in;

  // Set ans as identity matrix
  ans.clear();
  for (int i = 0; i < K; ++i)
    ans[i][i] = 1;

  while (n) {
    if (n & 1)
      ans = ans * b;

    n >>= 1;
    b = b * b;
  }

  return ans;
}


// Solves f(n) = x * f(n - 1) + y * f(n - 2)
matrix solve(ll x, ll y, ll n) {
  matrix in;

  in[0][0] = x % MOD;
  in[0][1] = y % MOD;
  in[1][0] = 1;
  in[1][1] = 0;

  return matrix_pow(in, n);
}
